@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2019 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2023 Savoir-faire Linux Inc.
  *  Author : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "rng.h"
 
 #include <msgpack.hpp>
+#include <fmt/core.h>
 
 #ifndef _WIN32
 #include <netinet/in.h>
@@ -40,10 +41,13 @@ typedef uint16_t in_port_t;
 #include <iomanip>
 #include <array>
 #include <vector>
+#include <string_view>
 #include <algorithm>
 #include <stdexcept>
 #include <sstream>
+
 #include <cstring>
+#include <cstddef>
 
 namespace dht {
 
@@ -54,9 +58,9 @@ namespace crypto {
 }
 
 /**
- * Represents an InfoHash.
- * An InfoHash is a byte array of HASH_LEN bytes.
- * InfoHashes identify nodes and values in the Dht.
+ * Represents an Hash,
+ * a byte array of N bytes.
+ * Hashes identify nodes and values in the Dht.
  */
 template <size_t N>
 class OPENDHT_PUBLIC Hash {
@@ -65,10 +69,10 @@ public:
     typedef typename T::iterator iterator;
     typedef typename T::const_iterator const_iterator;
 
-    Hash () noexcept {
+    Hash() noexcept {
         data_.fill(0);
     }
-    Hash (const uint8_t* h, size_t data_len) {
+    Hash(const uint8_t* h, size_t data_len) {
         if (data_len < N)
             data_.fill(0);
         else
@@ -79,7 +83,12 @@ public:
      * hex must be at least 2.HASH_LEN characters long.
      * If too long, only the first 2.HASH_LEN characters are read.
      */
-    explicit Hash(const std::string& hex);
+    explicit Hash(std::string_view hex) {
+        if (hex.size() < 2*N)
+            data_.fill(0);
+        else
+            fromString(hex.data());
+    }
 
     Hash(const msgpack::object& o) {
         msgpack_unpack(o);
@@ -93,14 +102,10 @@ public:
     iterator end() { return data_.end(); }
     const_iterator cend() const { return data_.cend(); }
 
+    static constexpr inline Hash zero() noexcept { return Hash{}; }
+
     bool operator==(const Hash& h) const {
-        auto a = reinterpret_cast<const uint32_t*>(data_.data());
-        auto b = reinterpret_cast<const uint32_t*>(h.data_.data());
-        constexpr unsigned n = N / sizeof(uint32_t);
-        for (unsigned i=0; i < n; i++)
-            if (a[i] != b[i])
-                return false;
-        return true;
+        return data_ == h.data_;
     }
     bool operator!=(const Hash& h) const { return !(*this == h); }
 
@@ -112,6 +117,14 @@ public:
         return false;
     }
 
+    Hash operator^(const Hash& o) const {
+        Hash result;
+        for(auto i = 0u; i < N; i++) {
+            result[i] = data_[i] ^ o.data_[i];
+        }
+        return result;
+    }
+
     explicit operator bool() const {
         auto a = reinterpret_cast<const uint32_t*>(data_.data());
         auto b = reinterpret_cast<const uint32_t*>(data_.data() + N);
@@ -142,10 +155,6 @@ public:
         return 8 * i + j;
     }
 
-    /**
-     * Forget about the ``XOR-metric''.  An id is just a path from the
-     * root of the tree, so bits are numbered from the start.
-     */
     static inline int cmp(const Hash& id1, const Hash& id2) {
         return std::memcmp(id1.data_.data(), id2.data_.data(), N);
     }
@@ -179,16 +188,12 @@ public:
     int
     xorCmp(const Hash& id1, const Hash& id2) const
     {
-        for(unsigned i = 0; i < N; i++) {
-            uint8_t xor1, xor2;
+        for (unsigned i = 0; i < N; i++) {
             if(id1.data_[i] == id2.data_[i])
                 continue;
-            xor1 = id1.data_[i] ^ data_[i];
-            xor2 = id2.data_[i] ^ data_[i];
-            if(xor1 < xor2)
-                return -1;
-            else
-                return 1;
+            uint8_t xor1 = id1.data_[i] ^ data_[i];
+            uint8_t xor2 = id2.data_[i] ^ data_[i];
+            return (xor1 < xor2) ? -1 : 1;
         }
         return 0;
     }
@@ -217,7 +222,7 @@ public:
         return v;
     }
 
-    static inline Hash get(const std::string& data) {
+    static inline Hash get(std::string_view data) {
         return get((const uint8_t*)data.data(), data.size());
     }
 
@@ -225,6 +230,11 @@ public:
         return get(data.data(), data.size());
     }
 
+    template <size_t H>
+    static Hash get(const Hash<H>& o) {
+        return get(o.data(), o.size());
+    }
+
     /**
      * Computes the hash from a given data buffer of size data_len.
      */
@@ -237,12 +247,17 @@ public:
 
     static Hash getRandom();
 
+    template <typename Rd>
+    static Hash getRandom(Rd&);
+
     template <size_t M>
     OPENDHT_PUBLIC friend std::ostream& operator<< (std::ostream& s, const Hash<M>& h);
 
     template <size_t M>
     OPENDHT_PUBLIC friend std::istream& operator>> (std::istream& s, Hash<M>& h);
 
+    /** Returns view to thread-allocated memory, only valid until the next call to this function. */
+    std::string_view to_view() const { return std::string_view(to_c_str(), N*2); }
     const char* to_c_str() const;
 
     std::string toString() const;
@@ -286,14 +301,6 @@ std::istream& operator>> (std::istream& s, Hash<N>& h)
     return s;
 }
 
-template <size_t N>
-Hash<N>::Hash(const std::string& hex) {
-    if (hex.size() < 2*N)
-        data_.fill(0);
-    else
-        fromString(hex.c_str());
-}
-
 template <size_t N>
 void
 Hash<N>::fromString(const char* in) {
@@ -316,7 +323,7 @@ Hash<N>
 Hash<N>::getRandom()
 {
     Hash h;
-    crypto::random_device rdev;
+    std::random_device rdev;
     std::uniform_int_distribution<uint32_t> rand_int;
     auto a = reinterpret_cast<uint32_t*>(h.data());
     auto b = reinterpret_cast<uint32_t*>(h.data() + h.size());
@@ -324,7 +331,20 @@ Hash<N>::getRandom()
     return h;
 }
 
-struct HexMap : public std::array<std::array<char, 2>, 256> {
+template <size_t N>
+template <typename Rd>
+Hash<N>
+Hash<N>::getRandom(Rd& rdev)
+{
+    Hash h;
+    std::uniform_int_distribution<uint32_t> rand_int;
+    auto a = reinterpret_cast<uint32_t*>(h.data());
+    auto b = reinterpret_cast<uint32_t*>(h.data() + h.size());
+    std::generate(a, b, std::bind(rand_int, std::ref(rdev)));
+    return h;
+}
+
+struct alignas(std::max_align_t) HexMap : public std::array<std::array<char, 2>, 256> {
     HexMap() {
         for (size_t i=0; i<size(); i++) {
             auto& e = (*this)[i];
@@ -338,11 +358,27 @@ private:
 
 OPENDHT_PUBLIC extern const HexMap hex_map;
 
+inline std::string
+toHex(const uint8_t* data, size_t size) {
+    std::string ret(size * 2, '\0');
+    for (size_t i=0; i<size; i++) {
+        auto b = ret.data()+i*2;
+        const auto& m = hex_map[data[i]];
+        *((uint16_t*)b) = *((uint16_t*)&m);
+    }
+    return ret;
+}
+
+inline std::string
+toHex(const std::vector<uint8_t>& data) {
+    return toHex(data.data(), data.size());
+}
+
 template <size_t N>
 const char*
 Hash<N>::to_c_str() const
 {
-    thread_local std::array<char, N*2+1> buf;
+    alignas(std::max_align_t) thread_local std::array<char, N*2+1> buf;
     for (size_t i=0; i<N; i++) {
         auto b = buf.data()+i*2;
         const auto& m = hex_map[data_[i]];
@@ -358,27 +394,11 @@ Hash<N>::toString() const
     return std::string(to_c_str(), N*2);
 }
 
-const InfoHash zeroes {};
-
-struct OPENDHT_PUBLIC NodeExport {
-    InfoHash id;
-    sockaddr_storage ss;
-    socklen_t sslen;
-
-    template <typename Packer>
-    void msgpack_pack(Packer& pk) const
-    {
-        pk.pack_map(2);
-        pk.pack(std::string("id"));
-        pk.pack(id);
-        pk.pack(std::string("addr"));
-        pk.pack_bin(sslen);
-        pk.pack_bin_body((char*)&ss, sslen);
-    }
-
-    void msgpack_unpack(msgpack::object o);
-
-    OPENDHT_PUBLIC friend std::ostream& operator<< (std::ostream& s, const NodeExport& h);
-};
-
 }
+
+template <size_t N>
+struct fmt::formatter<dht::Hash<N>>: formatter<string_view> {
+  constexpr auto format(const dht::Hash<N>& c, format_context& ctx) const {
+    return formatter<string_view>::format(c.to_view(), ctx);
+  }
+};
